import time
import typer
from typing_extensions import Annotated
from rich.progress import Progress, SpinnerColumn, TextColumn
from ..utils.modtools import split_mod
from ..utils.basetools import check_path_exists
from ..utils.abs_position import gene_feature_distance_calculator
from ..utils.modifications import form_reads_get_modifications

app = typer.Typer()

@app.command()
def detectMod(
    input: Annotated[str, typer.Option("--input", "-i", help="Input fastq files.")],
    sam: Annotated[str, typer.Option("--sam", "-s", help="mapping sam/bam file.")],
    bed: Annotated[str, typer.Option("--bed", "-b", help="bed file for transcripts sites.")],
    regions: Annotated[str, typer.Option("--regions", "-r", help="regions file for gene feature distance calculation.")],
    output: Annotated[str, typer.Option("--output", "-o", help="Output file path.")]=".",
    prefix: Annotated[str, typer.Option("--prefix", "-p", help="Prefix for output files.")]="prefix",
    pvalue: Annotated[float, typer.Option("--pvalue", help="pvalue cutoff for modification sites.")]=0.98,
    ):
    """
    detect modification sites in input fastq files.
    """
    
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        transient=True,
    ) as progress:
        progress.add_task(description="Detecting modification start...", total=None)
        start=time.time()
        
        output_file=f"{output}/{prefix}.bed"
        progress.add_task(description="Getting modification from fq files...", total=None)
        if not check_path_exists(output_file):
            mod = form_reads_get_modifications(input, sam, bed, output_file, pvalue)
            mod.get_mod_position_with_sam()
        progress.add_task(description=f"Getting modification from fq files Done", total=None)

        progress.add_task(description="Splitting modification sites...", total=None)
        if not check_path_exists(f"{output}/{prefix}_m6A.bed"):
            split_mod(output_file, prefix)
        progress.add_task(description="Splitting modification sites Done", total=None)

        progress.add_task(description="Calculating absolute distance...", total=None)
        bed_file_list = [f"{output}/{prefix}_m6A.bed", f"{output}/{prefix}_m5C.bed", f"{output}/{prefix}_psi.bed", f"{output}/{prefix}_AtoI.bed"]
        for bed_file in bed_file_list:
            gfc_output = bed_file.replace(".bed", "_abs_dist.txt")
            if not check_path_exists(gfc_output):
                gfc = gene_feature_distance_calculator(bed_file, regions, gfc_output)
                gfc.process_bed_file()
        progress.add_task(description="Calculating absolute distance Done", total=None)
        
        end=time.time()
        time_cost=f"{(end - start) // 3600}h{((end - start) % 3600) // 60}m{(end - start) % 60:.2f}s"
        print(f"Detecting modification sites Done, time cost: {time_cost}")
        progress.add_task(description=f"Detecting modification sites Done, time cost: {time_cost}", total=None)


#!/usr/bin/python3
# -*- coding: utf-8 -*-
__author__ = "legendzdy@dingtalk.com"
"""
Author: legendzdy@dingtalk.com
Data: 20250210
Description:
This Script of raw fq data from Legendzdy.  Of course, this means there's a possibility
for other ways.Use at your own discretion.
"""
#%%fowardDir
import argparse, re, gzip
import pysam

class getModifications(object):
    def __init__(self, inputFq, samFile, bedFile, output):
        self.inputFq = inputFq
        self.samFile = samFile
        self.bedFile = bedFile
        self.output = output
    
    def posFind(self, modList):
        counter = 0
        posList = []
        for mod in modList:
            counter += int(mod)
            posList.append(counter)
            counter += 1
        return posList
    
    def getAnnotation(self):
        annot = {}
        with open(self.bedFile, 'r') as f:
            for line in f:
                line = line.strip().split("\t")
                gid = line[0] + ":" + line[1]
                enst = line[3]
                base = line[4]
                annot[gid] = [enst, base]
        return annot
    
    def getModPositionWithReadAndGenome(self):
        samfile = pysam.AlignmentFile(self.samFile, "r")
        readAndGenome = {}
        for read in samfile:
            if not read.is_unmapped:
                genome_positions = []
                read_positions = []
                pos = read.reference_start
                read_pos = 1
                for op, length in read.cigartuples:
                    # print(f"op: {op}, length: {length}")
                    if op == 0:  # M: 匹配或错配
                        # 匹配的碱基在基因组上的位置
                        genome_positions.extend(range(pos, pos + length))
                        read_positions.extend(range(read_pos, read_pos + length))
                        pos += length
                        read_pos += length
                    elif op == 1:  # I: 插入
                        # 插入的碱基在参考基因组上不占位置，但在 read 上有位置
                        # read_positions.extend(range(read_pos, read_pos + length))
                        read_pos += length
                    elif op == 2:  # D: 删除
                        # 删除的碱基在参考基因组上仍占位置，但跳过它们
                        # genome_positions.extend(range(pos, pos + length))
                        pos += length
                    elif op == 4:  # S: 跳过
                        # 跳过的碱基不占基因组位置，但在 read 上有位置
                        # read_positions.extend(range(read_pos, read_pos + length))
                        read_pos += length
                    elif op == 5:  # H: 硬剪切
                        # 硬剪切不占基因组位置
                        read_pos += length
                    else:
                        print(f"op: {op}, length: {length} not supported")
                if read.is_reverse:
                    strand = "-"
                else:
                    strand = "+"
                readAndGenome[read.query_name] = [read_positions, genome_positions, read.reference_name, strand]
        return readAndGenome
    
    def intersectionPosition(self, mm, baseList, mlNum, refReadPos, refGenomePos):
        modList = mm.split(",")[1:]
        posList = self.posFind(modList)
        mlcount = 0
        readPos = []
        readPosPvalue = []
        for pos in posList:
            rpos = baseList[pos]
            pvalue = round(int(mlNum[mlcount])/255, 3)
            readPos.append(rpos)
            readPosPvalue.append(pvalue)
            mlcount += 1
        intersection = list(set(refReadPos) & set(readPos))
        if len(intersection) != 0:
            PosRead = [readPos.index(x) for x in intersection]
            PosGeno = [refReadPos.index(x) for x in intersection]
            targetPvalue = [readPosPvalue[i] for i in PosRead]
            targetGeno = [refGenomePos[i] for i in PosGeno]
        else:
            targetGeno = []
            targetPvalue = []
        return targetGeno, targetPvalue
    
    def getModPositionWithRead(self):
        annot = self.getAnnotation()
        readAndGenome = self.getModPositionWithReadAndGenome()
        with open(self.output, 'w') as out:
            abase = re.compile(r'A')
            tbase = re.compile(r'T')
            cbase = re.compile(r'C')
            with gzip.open(self.inputFq, 'rt') as f:
                while True:
                    try:
                        name = next(f).strip()
                        seq = next(f).strip()
                        plus = next(f).strip()
                        qual = next(f).strip()
                        
                        namelist = name.split("\t")
                        id = namelist[0].split("@")[-1]
                        if id in readAndGenome:
                            refReadPos = readAndGenome[id][0]
                            refGenomePos = readAndGenome[id][1]
                            chrom = readAndGenome[id][2]
                            strand = readAndGenome[id][3]
                            MMtagList = namelist[-2].split("MM:Z:")[-1].split(";")
                            # print(MMtagList)
                            MLtag = namelist[-1].split(",")[1:]
                            Alist = [match.start() + 1 for match in abase.finditer(seq)]
                            Tlist = [match.start() + 1 for match in tbase.finditer(seq)]
                            Clist = [match.start() + 1 for match in cbase.finditer(seq)]
                            mlstart = 0
                            for mm in MMtagList:
                                mmlen = len(mm.split(","))
                                if mmlen > 1:
                                    mlend = mlstart + mmlen - 1
                                    mlNum = MLtag[mlstart:mlend]
                                    mlstart += mmlen -1
                                    if mm.startswith('A+a'):
                                        targetGeno, targetPvalue = self.intersectionPosition(mm, Alist, mlNum, refReadPos, refGenomePos)
                                        if len(targetGeno) != 0:
                                            for i in range(len(targetGeno)):
                                                start = targetGeno[i]
                                                end = start + 1
                                                gid = "chr" + chrom + ":" + str(start)
                                                modType = "m6A"
                                                if gid in annot:
                                                    enst = annot[gid][0]
                                                    base = annot[gid][1]
                                                    out.write(f"chr{chrom}\t{start}\t{end}\t{targetPvalue[i]}\t{enst}\t{strand}\t{modType}\t{base}\n")
                                                else:
                                                    print(f"gid: {gid} not in annot")
                                    elif mm.startswith('A+17596') :
                                        targetGeno, targetPvalue = self.intersectionPosition(mm, Alist, mlNum, refReadPos, refGenomePos)
                                        if len(targetGeno) != 0:
                                            for i in range(len(targetGeno)):
                                                start = targetGeno[i]
                                                end = start + 1
                                                gid = "chr" + chrom + ":" + str(start)
                                                modType = "m1A"
                                                if gid in annot:
                                                    enst = annot[gid][0]
                                                    base = annot[gid][1]
                                                    out.write(f"chr{chrom}\t{start}\t{end}\t{targetPvalue[i]}\t{enst}\t{strand}\t{modType}\t{base}\n")
                                                else:
                                                    print(f"gid: {gid} not in annot")
                                    elif mm.startswith('T+17802'):
                                        targetGeno, targetPvalue = self.intersectionPosition(mm, Tlist, mlNum, refReadPos, refGenomePos)
                                        if len(targetGeno) != 0:
                                            for i in range(len(targetGeno)):
                                                start = targetGeno[i]
                                                end = start + 1
                                                gid = "chr" + chrom + ":" + str(start)
                                                modType = "psi"
                                                if gid in annot:
                                                    enst = annot[gid][0]
                                                    base = annot[gid][1]
                                                    out.write(f"chr{chrom}\t{start}\t{end}\t{targetPvalue[i]}\t{enst}\t{strand}\t{modType}\t{base}\n")
                                                else:
                                                    print(f"gid: {gid} not in annot")
                                    elif mm.startswith('C+m'):
                                        targetGeno, targetPvalue = self.intersectionPosition(mm, Clist, mlNum, refReadPos, refGenomePos)
                                        if len(targetGeno) != 0:
                                            for i in range(len(targetGeno)):
                                                start = targetGeno[i]
                                                end = start + 1
                                                gid = "chr" + chrom + ":" + str(start)
                                                modType = "m5C"
                                                if gid in annot:
                                                    enst = annot[gid][0]
                                                    base = annot[gid][1]
                                                    out.write(f"chr{chrom}\t{start}\t{end}\t{targetPvalue[i]}\t{enst}\t{strand}\t{modType}\t{base}\n")
                                                else:
                                                    print(f"gid: {gid} not in annot")         
                    except StopIteration:
                        break
                    
        
if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='modifacts detection from raw fastq data.'
    )
    parser.add_argument('--version', action='version', version='modkit Version: %(prog)s 0.23')
    parser.add_argument('-i', '--inputFq',  required=True, help='The input fastq file with MM tag.')
    parser.add_argument('-s', '--samFile',  required=True, help='The SAM file formed by mapping the input fastq file.')
    parser.add_argument('-b', '--bedFile',  required=True, help='The annotated BED file.')
    parser.add_argument('-o', '--output',  required=True, help='The output file.')
    args = parser.parse_args()
    mod = getModifications(args.inputFq, args.samFile, args.bedFile, args.output)
    mod.getModPositionWithRead()


import time
import typer
from typing_extensions import Annotated
from rich.progress import Progress, SpinnerColumn, TextColumn
from ..utils.map import minimap2map

app = typer.Typer()

@app.command()
def gene(
    input: Annotated[str, typer.Option("--input", "-i", help="Input fastq files.")],
    reference: Annotated[str, typer.Option("--reference", "-r", help="reference genome path.")],
    output: Annotated[str, typer.Option("--output", "-o", help="output for output sam/bam files.")],
    tool: Annotated[str, typer.Option("--tool", help="minimap2.")]="minimap2",
    parms: Annotated[str, typer.Option("--parms", help="minimap2 parameters for mapping.")]="--secondary=no --cs -a",
    threads: Annotated[int, typer.Option("--threads", "-t", help="Number of threads.")]=4,
    ):
    """
    Mapping of nanopore reads to a reference genome.
    """
    
    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        transient=True,
    ) as progress:
        try:
            progress.add_task(description="map reference...", total=None)
            start=time.time()
            minimap2map(input, reference, output, tool, parms, threads)
            end=time.time()
            time_cost=f"{(end - start) // 3600}h{((end - start) % 3600) // 60}m{(end - start) % 60:.2f}s"
            print(f"map reference Done, time cost: {time_cost}")
            progress.add_task(description=f"map reference Done, time cost: {time_cost}", total=None)
        except Exception as e:
            print(f"Error: {e}")
            progress.add_task(description="map reference Failed", total=None)
            exit(1)
    


import typer
import subprocess
from pathlib import Path
from rich.progress import track
from typing_extensions import Annotated
from ..utils.map import minimap2map

app = typer.Typer()

@app.command()
def isoformAS(
    input: Annotated[str, typer.Option("--input", "-i", help="Input fastq files.")],
    reference: Annotated[str, typer.Option("--reference", "-r", help="reference genome path.")],
    prefix: Annotated[str, typer.Option("--prefix", "-p", help="Prefix for output files.")],
    tool: Annotated[str, typer.Option("--tool", help="minimap2.")]="minimap2",
    parms: Annotated[str, typer.Option("--parms", help="minimap2 parameters for mapping.")]="--secondary=no --cs -a",
    threads: Annotated[int, typer.Option(help="Number of threads.")]=4,
    ):
    """
    alternative splicing analysis.
    """
    minimap2map(input, reference, prefix, tool, parms, threads)


#!/usr/bin/env python3
__author__ = "legendzdy@dingtalk.com"
"""
Author: legendzdy@dingtalk.com
Data: 20201224
Description:
This Script of ONTrnaDirectSeq from Legendzdy.  Of course, this means there's a possibility
for other ways.Use at your own discretion.
"""
import pandas as pd
import numpy as np
from sklearn.impute import SimpleImputer
import joblib
import argparse


def fetch_reads(sample, model):
    Snames = ["read", "UmisRate", "UmisQrate", "t_a_rate", "t_a_Qrate",
              "t_c_rate", "t_c_Qrate", "t_g_rate", "t_g_Qrate", "del_rate"]
    Sdf = pd.read_csv(sample)
    Sdf = Sdf[Snames]
    Sdf = Sdf.replace([np.inf, -np.inf], np.nan)
    Sdf.eval("""
    Um_UmQ = UmisRate*UmisQrate
    Um_t_a_rate = UmisRate*t_a_rate
    Um_t_a_Qrate = UmisRate*t_a_Qrate
    Um_t_c_rate = UmisRate*t_c_rate
    Um_t_c_Qrate = UmisRate*t_c_Qrate
    Um_t_g_rate = UmisRate*t_g_rate
    Um_t_g_Qrate = UmisRate*t_g_Qrate
    Um_del_rate = UmisRate*del_rate
    UQ_t_a_rate = UmisQrate*t_a_rate
    UQ_t_a_Qrate = UmisQrate*t_a_Qrate
    UQ_t_c_rate = UmisQrate*t_c_rate
    UQ_t_c_Qrate = UmisQrate*t_c_Qrate
    UQ_t_g_rate = UmisQrate*t_g_rate
    UQ_t_g_Qrate = UmisQrate*t_g_Qrate
    UQ_del_rate = UmisQrate*del_rate
    t_a_rate_t_a_Qrate = t_a_rate*t_a_Qrate
    t_a_rate_t_c_rate = t_a_rate*t_c_rate
    t_a_rate_t_c_Qrate = t_a_rate*t_c_Qrate
    t_a_rate_t_g_rate = t_a_rate*t_g_rate
    t_a_rate_t_g_Qrate = t_a_rate*t_g_Qrate
    t_a_rate_del_rate = t_a_rate*del_rate
    t_a_Qrate_t_c_rate = t_a_Qrate*t_c_rate
    t_a_Qrate_t_c_Qrate = t_a_Qrate*t_c_Qrate
    t_a_Qrate_t_g_rate = t_a_Qrate*t_g_rate
    t_a_Qrate_t_g_Qrate = t_a_Qrate*t_g_Qrate
    t_a_Qrate_del_rate = t_a_Qrate*del_rate
    t_c_rate_t_c_Qrate = t_c_rate*t_c_Qrate
    t_c_rate_t_g_rate = t_c_rate*t_g_rate
    t_c_rate_t_g_Qrate = t_c_rate*t_g_Qrate
    t_c_rate_del_rate = t_c_rate*del_rate
    t_c_Qrate_t_g_rate = t_c_Qrate*t_g_rate
    t_c_Qrate_t_g_Qrate = t_c_Qrate*t_g_Qrate
    t_c_Qrate_del_rate = t_c_Qrate*del_rate
    t_g_rate_t_g_Qrate = t_g_rate*t_g_Qrate
    t_g_rate_del_rate = t_g_rate*del_rate
    t_g_Qrate_del_rate = t_g_Qrate*del_rate
    """, inplace=True)
    S_test = Sdf.iloc[:, 1:]
    imputer = SimpleImputer(missing_values=np.nan, strategy='mean')
    S_test_t = imputer.fit_transform(S_test)
    clf2 = joblib.load(model)
    S_y = clf2.predict(S_test_t)
    dict, i = {}, 0
    while i < len(S_y):
        if int(S_y[i]) == 1:
            dict[Sdf["read"][i]] = 1
        i += 1
    return dict

def new_fq(sample, model, rawfq, newfq):
    dict_new = fetch_reads(sample, model)
    with open(newfq, 'w') as fo:
        with open(rawfq, 'r') as fi:
            while True:
                try:
                    line1 = next(fi).strip()
                    line2 = next(fi).strip()
                    line3 = next(fi).strip()
                    line4 = next(fi).strip()
                    if dict_new.get(line1.split()[0][1:], None) != None:
                        fo.write("{}\n{}\n{}\n{}\n".format(
                            line1, line2, line3, line4))
                except StopIteration:
                    break

def main():
    parser = argparse.ArgumentParser(
        description='Extract length and quality information of sequences form fastq file')
    parser.add_argument('-s', '--signal', type=str, required=False,
                        help='sample raw data fecth raw signal')
    parser.add_argument('-r', '--rawfq', type=str, required=True,
                        help='sample raw fastq data')
    parser.add_argument('-m', '--model', type=str, required=True,
                        help='train model')
    parser.add_argument('-o', '--out_file', type=str, required=True,
                        help='Path to an output file to be created,default= ./')
    args = parser.parse_args()
    new_fq(args.signal, args.model, args.rawfq, args.out_file)


if __name__ == '__main__':
    main()
# new_fq("OV-FTO-2-0603.fastq.sam.csv", "test.pkl", "all.fastq", "new.fastq")



# -*- coding: utf-8 -*-
__author__ = "legendzdy@dingtalk.com"
"""
Author: legendzdy@dingtalk.com
Data: 20250217
Description:
function basetools.
"""
import os

def check_path_exists(path):
    return os.path.exists(path)


# -*- coding: utf-8 -*-
__author__ = "legendzdy@dingtalk.com"
"""
Author: legendzdy@dingtalk.com
Data: 20250217
Description:
function map.
"""
import argparse

def split_mod(input, prefix, pvalue):
    """
    split modification bed file into single bed files.
    Args:
        input: input modification bed file.
        prefix: output file prefix.
    Returns:
        None.
    """
    
    with open(input, 'r') as file:
        dupfile = {}
        for line in file:
            count = 1
            fields = line.strip().split("\t")
            if float(fields[3]) >= float(pvalue):
                print(float(fields[3]))
                if dupfile.get(fields[4]) is None:
                    dupfile[fields[4]] = fields + [str(count)]
                else:
                    count += int(dupfile[fields[4]][-1])
                    dupfile[fields[4]][-1] = str(count)
    
        for key, values in dupfile.items():
            output_filename = prefix + "_" + values[6] + ".bed" 
            with open(output_filename, 'a') as output_file:
                output_file.write("\t".join(values) + "\n")

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='Parallel modifacts detection from raw fastq data.'
    )
    parser.add_argument('--version', action='version', version='modkit 1.0')
    parser.add_argument('-i', '--input', required=True)
    parser.add_argument('-p', '--prefix', required=True)
    parser.add_argument('-v', '--pvalue', required=True, type=float)
    args = parser.parse_args()
    split_mod(args.input, args.prefix, args.pvalue)